@@ -112,7 +112,8 @@ namespace BitB
         enum ModemStatus
         { //{{{
 
-            FTDI_MS_LOW = 0x02, // Lower 4 bits are reserved, must be 0x02
+            FTDI_MAX64  = 0x01, // wMaxPacketSize 64
+            FTDI_MAX512 = 0x02, // wMaxPacketSize 512
             FTDI_CTS    = 0x10, // Clear to send
             FTDI_DSR    = 0x20, // Data set ready
             FTDI_RI     = 0x40, // Ring indicator
@@ -209,6 +210,8 @@ namespace BitB
         size_t                              m_chunklen;
         uint8_t                            *m_chunkbuf;
 
+        uint8_t                             m_expect_modemstatus;
+
 
     protected:
 
@@ -467,8 +470,7 @@ namespace BitB
                     switch( packethead )
                     {
                         case 0:
-                            if( __builtin_expect(
-                                m_chunkbuf[m_chunkhead] != (FTDI_DSR | FTDI_CTS | FTDI_MS_LOW), 0) )
+                            if( __builtin_expect(m_chunkbuf[m_chunkhead] != m_expect_modemstatus, 0) )
                             { //{{{
 
                                 size_t  chunklen = m_chunklen;
@@ -576,9 +578,8 @@ namespace BitB
 
                 if( __builtin_expect(xfer == 2, 0) )
                 {
-                    if( __builtin_expect(
-                        m_chunkbuf[0] != (FTDI_DSR | FTDI_CTS | FTDI_MS_LOW)
-                     || (m_chunkbuf[1] & ~(FTDI_THRE | FTDI_TEMT)) != 0, 0) )
+                    if( __builtin_expect(m_chunkbuf[0] != m_expect_modemstatus
+                                     || (m_chunkbuf[1] & ~(FTDI_THRE | FTDI_TEMT)) != 0, 0) )
                         ThrowError( _("FTDI: read invalid packet: len %5zu, got %5zu [%s ]"),
                                         len, xfer,
                                         OctetsToHex( OctetString( m_chunkbuf,
@@ -900,6 +901,9 @@ namespace BitB
                 m_maxpacket = alt.endpoint[0].wMaxPacketSize;
                 m_epin      = alt.endpoint[0].bEndpointAddress;
                 m_epout     = alt.endpoint[1].bEndpointAddress;
+
+                m_expect_modemstatus = FTDI_DSR | FTDI_CTS
+                                       | (m_maxpacket == 64 ? FTDI_MAX64 : FTDI_MAX512);
             }
             catch( const std::exception &e )
             {
