@@ -4,7 +4,7 @@
 //! @ingroup JsonParsing
 //! @brief Parser for JSON formatted data.
 //
-//  Copyright 2013 - 2015,  Ron <ron@debian.org>
+//  Copyright 2013 - 2016,  Ron <ron@debian.org>
 //  This file is distributed as part of the bit-babbler package.
 //
 ////////////////////////////////////////////////////////////////////
@@ -19,6 +19,11 @@
 #include <list>
 #include <stdexcept>
 #include <climits>
+#include <locale.h>
+
+#if HAVE_XLOCALE_H
+ #include <xlocale.h>
+#endif
 
 // We don't hash the Object member map by default right now.
 // Most of the structures we are expecting to handle at this stage will all be
