@@ -452,6 +452,15 @@ namespace BitB
             Result          result[ DATASET_MAX ];
             Fail            fail;
 
+            // Enable extra code to report debug information when the long term
+            // normalisation occurs.  Note that you could be waiting for a very
+            // long time for that to happen if size_t is a 64-bit type ...
+            //#define CHECK_NORMALISATION
+
+           #ifdef CHECK_NORMALISATION
+            double          skew;
+           #endif
+
 
             Data()
             {
@@ -480,6 +489,10 @@ namespace BitB
 
                 fail = Fail( data["Failed"] );
 
+               #ifdef CHECK_NORMALISATION
+                skew = 0.0;
+               #endif
+
             } //}}}
 
             Data( Results_Only_, const Json::Data::Handle &data )
@@ -514,6 +527,10 @@ namespace BitB
                 corr2      = 0.0;
                 corr3      = 0.0;
 
+               #ifdef CHECK_NORMALISATION
+                skew       = 0.0;
+               #endif
+
             } //}}}
 
             void normalise_long_term()
@@ -521,15 +538,74 @@ namespace BitB
 
                 if( samples > size_t(-1) / 2 )
                 {
+                   #ifdef CHECK_NORMALISATION
+                    Log<0>( "Before:\n%s\n", ReportBinsByFreq(5,5).c_str() );
+                    Log<0>( "s: %zu, ir: %zu, ps: %zu, c1: %f, c2: %f, c3: %f\n",
+                            samples, inradius, pisamples, corr1, corr2, corr3 );
+                   #endif
+
+
+                    double  old_samples  = samples;
+                    double  old_expected = old_samples / NBINS;
+                    double  new_expected = old_expected / 2.0;
+
+                    samples = 0;
+
                     for( size_t i = 0; i < NBINS; ++i )
-                        bin[i] >>= 1;
+                    {
+                        double error = bin[i] - old_expected;
+                        double chisq = (error * error) / old_expected;
+                        double fudge = sqrt(new_expected * chisq);
+
+                        if( error < 0 )
+                            bin[i] = lrint(new_expected - fudge);
+                        else
+                            bin[i] = lrint(new_expected + fudge);
+
+                        samples += bin[i];
+                    }
+
+                    double  scale = samples / old_samples;
+
+                    corr1 *= scale;
+                    corr2 *= scale;
+                    corr3 *= scale;
+
+
+                   #ifdef CHECK_NORMALISATION
+                    Log<0>( "After:\n%s\n", ReportBinsByFreq(5,5).c_str() );
+                    Log<0>( "s: %zu, ir: %zu, ps: %zu, c1: %f, c2: %f, c3: %f\n",
+                            samples, inradius, pisamples, corr1, corr2, corr3 );
+
+                    skew += samples - (old_samples / 2.0);
+
+                    Log<0>( "Count skew: %0.1f (%0.15f), total %0.1f\n",
+                            samples - (old_samples / 2.0), scale, skew );
+
+                    Log<0>( "Before: %s\n", result[CURRENT].Report().c_str() );
+                    ComputeResult();
+                    Log<0>( "After : %s\n", result[CURRENT].Report().c_str() );
+                   #endif
+                }
+
+                if( pisamples > size_t(-1) / 2 )
+                {
+                   #ifdef CHECK_NORMALISATION
+                    Log<0>( "s: %zu, ir: %zu, ps: %zu\n", samples, inradius, pisamples );
+                   #endif
+
 
-                    samples     >>= 1;
                     inradius    >>= 1;
                     pisamples   >>= 1;
-                    corr1       /= 2.0;
-                    corr2       /= 2.0;
-                    corr3       /= 2.0;
+
+
+                   #ifdef CHECK_NORMALISATION
+                    Log<0>( "s: %zu, ir: %zu, ps: %zu\n", samples, inradius, pisamples );
+
+                    Log<0>( "Before: %s\n", result[CURRENT].Report().c_str() );
+                    ComputeResult();
+                    Log<0>( "After : %s\n", result[CURRENT].Report().c_str() );
+                   #endif
                 }
 
             } //}}}
@@ -587,6 +663,61 @@ namespace BitB
 
             } //}}}
 
+            void ComputeResult()
+            { //{{{
+
+                double expected = double(samples) / NBINS;
+                double entropy  = 0.0;
+                double chisq    = 0.0;
+                double sum      = 0.0;
+
+                size_t cmax     = 0;
+                double pmax     = 0.0;
+
+                for( size_t i = 0; i < NBINS; ++i )
+                {
+                    double  error = bin[i] - expected;
+                    double  p = double(bin[i]) / samples;
+
+                    if( bin[i] > cmax )
+                    {
+                        cmax = bin[i];
+                        pmax = p;
+                    }
+
+                    if( p > 0.0 )
+                        entropy -= p * log2( p );
+
+                    chisq   += (error * error) / expected;
+                    sum     += double(bin[i]) * i;
+                }
+
+                // The autocorrelation coefficient for N samples at lag h is:
+                //   Rh = Ch / C0
+                //
+                // Where Ch is the autocovariance function:
+                //   Ch = 1/N * Sum( t=1 -> N-h, (Yt - Ybar)(Ytplush - Ybar) )
+                //
+                // And C0 is the variance function:
+                //   C0 = 1/N * Sum( t=1 -> N, (Yt - Ybar)^2 )
+                //
+                // where Ybar is the sample mean and Ytplush is the sample that
+                // is h samples after Yt (because ascii).  Here we only compute
+                // it for h=1.
+                //
+                // Rh ranges between -1.0 and 1.0, with 0 finding no correlation.
+
+                double  c1 = corr1 + corrn * corr0;
+                double  c2 = corr2 * corr2;
+                double  ac = (samples * c1 - c2) / (samples * corr3 - c2);
+
+                AddResult( entropy, chisq, sum / samples, 4.0 * inradius / pisamples,
+                           // report 1.0 if samples*c3-c2 == 0
+                           std::isfinite(ac) ? ac : 1.0,
+                           // NIST SP 800-90B Section 9.2, min-entropy of IID sources
+                           -log2( (cmax + 2.3 * sqrt(samples * pmax * (1 - pmax))) / samples ) );
+            } //}}}
+
 
             std::string ReportResult( DataSet set = CURRENT ) const
             {
@@ -716,60 +847,6 @@ namespace BitB
 
 
 
-        void compute_result( Data &d )
-        { //{{{
-
-            double expected = double(d.samples) / NBINS;
-            double entropy  = 0.0;
-            double chisq    = 0.0;
-            double sum      = 0.0;
-
-            size_t cmax     = 0;
-            double pmax     = 0.0;
-
-            for( size_t i = 0; i < NBINS; ++i )
-            {
-                double  error = d.bin[i] - expected;
-                double  p = double(d.bin[i]) / d.samples;
-
-                if( d.bin[i] > cmax )
-                {
-                    cmax = d.bin[i];
-                    pmax = p;
-                }
-
-                if( p > 0.0 )
-                    entropy -= p * log2( p );
-
-                chisq   += (error * error) / expected;
-                sum     += double(d.bin[i]) * i;
-            }
-
-            // The autocorrelation coefficient for N samples at lag h is:
-            //   Rh = Ch / C0
-            //
-            // Where Ch is the autocovariance function:
-            //   Ch = 1/N * Sum( t=1 -> N-h, (Yt - Ybar)(Ytplush - Ybar) )
-            //
-            // And C0 is the variance function:
-            //   C0 = 1/N * Sum( t=1 -> N, (Yt - Ybar)^2 )
-            //
-            // where Ybar is the sample mean and Ytplush is the sample that
-            // is h samples after Yt (because ascii).  Here we only compute
-            // it for h=1.
-            //
-            // Rh ranges between -1.0 and 1.0, with 0 finding no correlation.
-
-            double  corr1 = d.corr1 + d.corrn * d.corr0;
-            double  corr2 = d.corr2 * d.corr2;
-            double  ac = (d.samples * corr1 - corr2) / (d.samples * d.corr3 - corr2);
-
-            d.AddResult( entropy, chisq, sum / d.samples, 4.0 * d.inradius / d.pisamples,
-                         std::isfinite(ac) ? ac : 1.0,  // report 1.0 if samples*c3-c2 == 0
-                        // NIST SP 800-90B Section 9.2, min-entropy of IID sources
-                        -log2( (cmax + 2.3 * sqrt(d.samples * pmax * (1 - pmax))) / d.samples ) );
-        } //}}}
-
         // This one always operates on 8 bit samples, even in 16-bit mode
         void analyse_monte( const uint8_t *buf, size_t len )
         { //{{{
@@ -883,8 +960,8 @@ namespace BitB
             m_long.pisamples += m_short.pisamples;
             m_long.samples   += m_short.samples;
 
-            compute_result( m_short );
-            compute_result( m_long );
+            m_short.ComputeResult();
+            m_long.ComputeResult();
 
             m_long.normalise_long_term();
             m_previous_short = m_short;
