@@ -1667,6 +1667,9 @@ namespace BitB
             Log<0>( "vendorid: %s, productid: %s, serial: %s\n", vendorid, productid, serial );
 
 
+            // This function was added in udev release 167.
+           #if HAVE_UDEV_DEVICE_GET_SYSATTR_LIST_ENTRY
+
             struct udev_list_entry  *attrlist, *a;
 
             attrlist = udev_device_get_sysattr_list_entry( d );
@@ -1683,6 +1686,8 @@ namespace BitB
                 Log<0>( "  attribute %s = '%s'\n", key, val );
             }
 
+           #endif
+
 
             struct udev_list_entry  *proplist, *p;
 
@@ -1701,6 +1706,9 @@ namespace BitB
             }
 
 
+            // This function was added in udev release 154.
+           #if HAVE_UDEV_DEVICE_GET_TAGS_LIST_ENTRY
+
             struct udev_list_entry  *taglist, *t;
 
             taglist = udev_device_get_tags_list_entry( d );
@@ -1717,6 +1725,8 @@ namespace BitB
                 Log<0>( "  tag       %s = '%s'\n", key, val );
             }
 
+           #endif
+
         } //}}}
 
 
