@@ -738,7 +738,7 @@ namespace BitB
             { //{{{
 
                 // Let Device get at the private constructor
-                friend class Device;
+                friend struct Device;
 
             private:
 
